"""Console script for drugintfinder."""
import sys

import click

from drugintfinder.ranker import Ranker
from drugintfinder.utils import export_table
from drugintfinder.finder import InteractorFinder


@click.group(help=f"Druggable Interactor Finder Framework Command Line Utilities on {sys.executable}")
@click.version_option()
def main():
    """Console script for drugintfinder."""
    pass


@main.command()
@click.argument('name')
@click.option('-n', '--node-type', default='protein', help="Target node type. Defaults to 'protein'.")
@click.option('-e', '--edge-type', default='causal', help="Interactor/target relationship type. Defaults to 'causal'.")
@click.option('-m', '--pmods', default=[], help="Comma separated list of acceptable target protein modifications.")
@click.option('-d', '--druggable', is_flag=True, default=False, help="Flag to enable filtering of druggable ints.")
@click.option('-s', '--sql', is_flag=True, default=False, help="Flag to print query.")
@click.option('-o', '--output', default=None, help="Results output path - defaults to Excel.")
@click.option('-v', '--verbose', is_flag=True, default=False, help="Flag to print results to STDOUT.")
def find(
        name: str, node_type: str, edge_type: str, pmods: str, druggable: bool, sql: bool, output: str, verbose: bool
):
    """Identify interactors of given target and criteria.

    Parameters
    ----------
    name: str
        Gene symbol or namespace value of the target node.
    node_type: str
        Type of target node to consider (e.g. 'protein', 'rna', 'gene', etc.)
    edge_type: str
        Edge type between interactor and target nodes (e.g. 'increases', 'causal', 'correlative', 'E' for all, etc.)
    pmods: str
        Comma separated list of 3-letter target protein modifications. This will filter target node results for those
        with specified pmods.
    druggable: bool
        If True, will only include interactors that are targeted by a drug in the graph.
    sql: bool
        Flag to print query.
    output: str
        Path to write results to. Currently only supports Excel.
    verbose: bool
        Flag to print results table to STDOUT.
    """
    if isinstance(pmods, str):
        pmods = pmods.split(",")

    finder = InteractorFinder(
        node_name=name, pmods=pmods, neighbor_edge_type=edge_type, node_type=node_type, print_sql=sql
    )
    if not druggable:
        finder.find_interactors()

    else:
        finder.druggable_interactors()

    if output:
        export_table(finder.results, output)

    if verbose:
        click.echo(finder.results)


@main.command()
@click.argument('name')
@click.option('-n', '--node-type', default='protein', help="Target node type. Defaults to 'protein'.")
@click.option('-m', '--pmods', default=[], help="Comma separated list of acceptable target protein modifications.")
@click.option('-r', '--reward', default=1, help="Points awarded for passing inspection criteria.")
@click.option('-p', '--penalty', default=-1, help="Points penalized for failing inspection criteria.")
@click.option('-o', '--output', default=None, help="Results output path - defaults to Excel.")
@click.option('-t', '--pivot', is_flag=True, default=False, help="Flag to enable target focused summary results.")
@click.option('-v', '--verbose', is_flag=True, default=False, help="Flag to print results to STDOUT.")
def rank(name: str, node_type: str, pmods: str, reward: str, penalty: str, output: str, verbose: bool, pivot: bool):
    """Ranks the drug/interactor combos with metadata and returns a summary table.

    Parameters
    ----------
    name: str
        Gene symbol or namespace value of the target node.
    node_type: str
        Type of target node to consider (e.g. 'protein', 'rna', 'gene', etc.)
    pmods: str
        Comma separated list of 3-letter target protein modifications. This will filter target node results for those
        with specified pmods.
    reward: str
        Number of points to reward entries that follow desired outcome.
    penalty: str
        Number of points to penalize entries that follow desired outcome.
    output: str
        Path to write results to. Currently only supports Excel.
    pivot: bool
        Flag to enable target focused summary results.
    verbose: bool
        Flag to print results table to STDOUT.
    """
    if isinstance(pmods, str):
        pmods = pmods.split(",")

    ranker = Ranker(name=name, node_type=node_type, pmods=pmods, reward=int(reward), penalty=int(penalty))
    ranker.rank()
    summary = ranker.summarize(pivot=pivot)

    if output:
        export_table(summary, output)

    if verbose:
        click.echo(summary)


if __name__ == "__main__":
    sys.exit(main())



"""Tests for `Ranker` class."""
import pandas as pd
import pytest

from drugintfinder.ranker import Ranker
from .constants import MAPT, PHOSPHORYLATION


@pytest.fixture
def ranker():
    """Create the base Ranker object for testing."""
    ranker = Ranker(name=MAPT, pmods=[PHOSPHORYLATION], print_sql=False)
    return ranker


class TestRanker:
    """Tests for the Ranker class."""

    def test_rank(self, ranker):
        """Test the find_interactors method."""
        ranker.rank()
        summary = ranker.summarize()

        assert isinstance(summary, pd.DataFrame)
        assert len(summary) >= 847

        expected_cols = ['Drug', 'Target', 'Synergizes', 'Number of BioAssays for Target',
                         'Number of Causal Edges for Target', 'Drug Patent Ongoing',
                         'Generic Version of Drug Available', 'Number of Drug Targets']
        assert all([col in summary.columns for col in expected_cols])

# TODO finish writing tests



"""Tests for `InteractorFinder` class."""
import pandas as pd
import pytest

from drugintfinder.finder import InteractorFinder
from .constants import CD33, PROTEIN, PHOSPHORYLATION, CAUSAL


@pytest.fixture
def finder():
    """Create reusable interactor finder object."""
    finder = InteractorFinder(
        node_name=CD33, pmods=[PHOSPHORYLATION], neighbor_edge_type=CAUSAL, node_type=PROTEIN, print_sql=False
    )
    return finder


class TestInteractorFinder:
    """Tests for the InteractorFinder class."""

    def test_find_interactors(self, finder):
        """Test the find_interactors method."""
        finder.find_interactors()
        results = finder.results

        assert results is not None
        assert isinstance(results, pd.DataFrame)
        assert len(results) >= 11

        expected_cols = ["target_species", "pmid", "pmc", "interactor_type", "interactor_name", "interactor_bel",
                         "relation_type", "target_bel", "target_type", "target_symbol", "pmod_type"]

        assert all([col in results.columns for col in expected_cols])

    def test_druggable_interactors(self, finder):
        """Test the druggable_interactors method."""
        finder.druggable_interactors()
        results = finder.results

        assert results is not None
        assert isinstance(results, pd.DataFrame)
        assert len(results) >= 70

        expected_cols = ['drug', 'capsule_interactor_type', 'capsule_interactor_bel', 'interactor_bel',
                         'interactor_type', 'interactor_name', 'relation_type', 'target_bel', 'target_symbol',
                         'target_type', 'pmid', 'pmc', 'rel_pub_year', 'rel_rid', 'drug_rel_rid', 'drug_rel_actions',
                         'drugbank_id', 'chembl_id', 'pubchem_id', 'pmod_type']

        assert all([col in results.columns for col in expected_cols])

    def test_unique_interactors(self, finder):
        """Test the unique_interactors method."""
        finder.druggable_interactors()

        ui = finder.unique_interactors()
        assert isinstance(ui, tuple)
        assert len(ui) >= 4

    def test_unique_drugs(self, finder):
        """Test the unique_drugs method."""
        finder.druggable_interactors()

        ud = finder.unique_drugs()
        assert isinstance(ud, tuple)
        assert len(ud) >= 20



import os
import sys
sys.path.insert(0, os.path.abspath('../../drugintfinder'))

# -- General configuration ---------------------------------------------
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.intersphinx',
    'sphinx_click.ext',
]

templates_path = ['_templates']

source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'Druggable Interactor Finder'
copyright = "2021, Bruce Schultz"
author = "Bruce Schultz"

# The version info for the project you're documenting, acts as replacement
# for |version| and |release|, also used in various other places throughout
# the built documents.
#
# The short X.Y version.
version = '0.1'

# The full version, including alpha/beta/rc tags.
release = '0.2.3'

language = 'en'
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']
pygments_style = 'sphinx'
todo_include_todos = False

intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
    'sqlalchemy': ('https://docs.sqlalchemy.org/en/14/', None),
}

# -- Options for HTML output -------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'
html_static_path = []
htmlhelp_basename = 'drugintfinderdoc'


# -- Options for LaTeX output ------------------------------------------

latex_elements = {}
latex_documents = [
    (master_doc, 'drugintfinder.tex',
     'Druggable Interactor Finder Documentation',
     'Bruce Schultz', 'manual'),
]


# -- Options for manual page output ------------------------------------
man_pages = [
    (master_doc, 'drugintfinder',
     'Druggable Interactor Finder Documentation',
     [author], 1)
]

texinfo_documents = [
    (master_doc, 'drugintfinder',
     'Druggable Interactor Finder Documentation',
     author,
     'drugintfinder',
     'One line description of project.',
     'Miscellaneous'),
]



"""Defined string constants."""

# Strings
POINTS = 'points'
PATENTS = 'patents'
IDENTIFIERS = 'identifiers'
PRODUCTS = 'products'
INTERACTORS = 'interactors'
DRUG_NAME = 'drug_name'
SYNERGY = 'synergy'
TRIALS = 'trials'
TARGET_COUNT = 'target_count'
CLINICAL_TRIALS = 'clinical_trials'
TIC = 'target_interactor_contradiction'
DAC = 'drug_action_contradiction'

# MAPPER
POSREG = 'positive_regulator'
NEGREG = 'negative_regulator'
NEUTRAL = 'neutral'

# API
PUBCHEM_BIOASSAY_API = "https://pubchem.ncbi.nlm.nih.gov/rest/pug/assay/target/accession/{}/aids/TXT"

###########
# Mappers #
###########

# Edge types
COMPILER = ['has_modification', 'has_product', 'reactant_in', 'acts_in', 'has_variant', 'translocates', 'includes']
CAUSAL = ['increases', 'decreases', 'directly_increases', 'directly_decreases', 'causes_no_change',
          'rate_limiting_step_of', 'regulates']
CORRELATIVE = ['association', 'no_correlation', 'positive_correlation', 'negative_correlation']
OTHER = ['has_member', 'has_members', 'has_component', 'has_components', 'equivalent_to', 'is_a', 'sub_process_of',
         'analogous_to', 'biomarker_for', 'prognostic_biomarker_for']
GENOMIC = ['transcribed_to', 'translated_to', 'orthologous']
BEL_RELATION = CAUSAL + CORRELATIVE + OTHER + GENOMIC
ALL = BEL_RELATION + COMPILER

EDGE_MAPPER = {
    'bel_relation': BEL_RELATION,
    'causal': CAUSAL,
    'correlative': CORRELATIVE,
    'other': OTHER,
    'genomic': GENOMIC,
    'compiler': COMPILER,
    'E': ALL,
}

CT_MAPPER = {
    'ongoing': [
        "Recruiting",
        "Enrolling by invitation",
        "Active, not recruiting",
        "Approved for marketing",
        "Available",
    ],
    'finished': [
        "Completed",
        "Unknown status",
        "Terminated",
        "Withdrawn",
        "Suspended",
        "No longer available",
        "Withheld",
        "Not yet recruiting",
        "Temporarily not available",
    ],
}

ACTION_MAPPER = {'activator': POSREG,
                 'aggregation inhibitor': NEGREG,
                 'agonist': POSREG,
                 'allosteric modulator': NEUTRAL,  # Can't tell if it's pos or neg
                 'antagonist': NEGREG,
                 'antibody': NEUTRAL,  # Can't tell if it's pos or neg
                 'binder': NEUTRAL,  # Can't tell if it's pos or neg
                 'binding': NEUTRAL,
                 'blocker': NEGREG,
                 'chaperone': POSREG,
                 'cofactor': POSREG,  # Cofactor = component for activity
                 'inducer': POSREG,
                 'inhibitor': NEGREG,
                 'ligand': NEUTRAL,  # Can't tell if it's agonist or anatagnoist
                 'modulator': NEUTRAL,  # Can't tell if it's pos or neg
                 'multitarget': NEGREG,  # 'multitarget' only used for Dasatinib which is an inhibitor
                 'negative modulator': NEGREG,
                 'neutralizer': NEGREG,
                 'other': NEUTRAL,
                 'other/unknown': NEUTRAL,
                 'partial agonist': POSREG,
                 'positive allosteric modulator': POSREG,
                 'potentiator': NEGREG,  # 'potentiator' only used for Pimecrolimus which is an inhibitor
                 'stabilization': POSREG,
                 'stimulator': POSREG,
                 'substrate': NEUTRAL,  # Can't tell if it's pos or neg
                 'suppressor': NEGREG,
                 'unknown': NEUTRAL,
                 'weak inhibitor': NEGREG,
                 }

###########
# Queries #
###########

IN_COUNT = "SELECT count(*) as number FROM causal WHERE in.name = '{}' AND in.pure = true AND in.@class = 'protein'"
OUT_COUNT = """SELECT count(*) as number FROM causal
WHERE out.name = '{}' AND out.pure = true AND out.@class = 'protein'"""

UNIPROT_ID = "SELECT uniprot.id FROM protein WHERE name = '{}' AND namespace = 'HGNC' and pure = true LIMIT 1"

CLINICAL_TRIALS_COUNT = "SELECT count(*) as trial_count FROM clinical_trial"
CLINICAL_TRIALS_DATA = "SELECT overall_status, trial_id, condition, mesh_conditions, drugs_in_trial FROM " \
                       "clinical_trial SKIP {} LIMIT {}"


DRUG_COUNT = "SELECT count(*) as num_drugs FROM drugbank"
DRUG_METADATA = "SELECT id as drugbank_id, name as drug_name, patents as drug_patents, " \
                "products.product as drug_products, target_symbols, clinical_trials.trial_id AS clinical_trials " \
                "FROM drugbank SKIP {} LIMIT {}"

ASSOCIATED_PATHWAYS = "SELECT count(*) FROM pathway_interaction WHERE out.name = '{}' OR in.name = '{}'"

INTERACTOR_QUERY = """MATCH {{class:pmod, as:pmod{}}}<-has__pmod-
{{class:{}, as:target, WHERE:(name = '{}')}}
.inE(){{class:{} ,as:relation, where:(@class != 'causes_no_change'{})}}
.outV(){{class:bel, as:interactor}}
RETURN
pmod.type as pmod_type,
relation.@class as relation_type,
target.name as target_symbol,
target.bel as target_bel,
target.@class as target_type,
interactor.bel as interactor_bel,
interactor.name as interactor_name,
interactor.@class as interactor_type,
relation.pmid as pmid,
relation.pmc as pmc,
target.species as target_species
"""

PURE_DRUGGABLE_QUERY = """MATCH {{class:pmod, as:pmod{}}}<-has__pmod-
{{class:{}, as:target, WHERE:(name = '{}')}}
.inE(){{class:causal,as:relation, where:(@class != 'causes_no_change'{})}}
.outV(){{class:bel, as:interactor}}
.inE(){{class:has_drug_target, as:drug_rel}}
.outV(){{class:drug, as:drug}}
RETURN
pmod.type as pmod_type,
relation.@class as relation_type,
relation.citation.pub_date.subString(0, 4) as rel_pub_year,
target.name as target_symbol,
target.bel as target_bel,
target.@class as target_type,
interactor.bel as interactor_bel,
interactor.name as interactor_name,
interactor.@class as interactor_type,
drug.label as drug,
drug.drugbank_id as drugbank_id,
drug.drugbank.chembl_id as chembl_id,
drug.drugbank.pubchem_cid as pubchem_id,
relation.pmid as pmid,
relation.pmc as pmc,
relation.@rid.asString() as rel_rid,
drug_rel.@rid.asString() as drug_rel_rid,
drug_rel.actions as drug_rel_actions
"""

CAPSULE_PREFIX = """MATCH {{class:pmod, as:pmod{}}}<-has__pmod-
{{class:{}, as:target, WHERE:(name = '{}')}}
.inE(){{class:causal,as:relation, where:(@class != 'causes_no_change')}}
.outV(){{class:bel, as:capsule_interactor}}"""

CAPSULE_MODIFIED_PROTEIN = """
.inE('has_modified_protein', 'has_variant_protein', 'has_located_protein', 'has_fragmented_protein')
.outV(){{class:protein, as:pure_interactor, WHERE:(pure=true)}}"""

CAPSULE_HAS_PROTEIN = """.outE('has__protein')
.inV(){{class:protein, as:pure_interactor, WHERE:(pure=true)}}"""

CAPSULE_SUFFIX = """.inE(){{class:has_drug_target, as:drug_rel}}
.outV(){{class:drug, as:drug}}
RETURN
pmod.type as pmod_type,
drug.label as drug,
drug.drugbank_id as drugbank_id,
drug.drugbank.chembl_id as chembl_id,
drug.drugbank.pubchem_cid as pubchem_id,
pure_interactor.@class as interactor_type,
pure_interactor.bel as interactor_bel,
pure_interactor.name as interactor_name,
capsule_interactor.bel as capsule_interactor_bel,
capsule_interactor.@class as capsule_interactor_type,
relation.@class as relation_type,
relation.citation.pub_date.subString(0, 4) as rel_pub_year,
target.name as target_symbol,
target.bel as target_bel,
target.@class as target_type,
relation.pmid as pmid,
relation.pmc as pmc,
relation.@rid.asString() as rel_rid,
drug_rel.@rid.asString() as drug_rel_rid,
drug_rel.actions as drug_rel_actions
"""

CAPSULE_DRUGGABLE_MODIFIED = CAPSULE_PREFIX + CAPSULE_MODIFIED_PROTEIN + CAPSULE_SUFFIX
CAPSULE_DRUGGABLE_COMPLEX = CAPSULE_PREFIX + CAPSULE_HAS_PROTEIN + CAPSULE_SUFFIX



"""Populate the database with the base information for faster querying."""
import logging

from tqdm import tqdm
from sqlalchemy import func
from datetime import datetime
from typing import List, Union, Dict
from ebel_rest import query as rest_query

from drugintfinder.constants import DRUG_COUNT, DRUG_METADATA, CLINICAL_TRIALS_COUNT, CLINICAL_TRIALS_DATA
from drugintfinder.defaults import session
from drugintfinder.models import Trials, Patents, Products, Drugs

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

sess = session()


def populate():
    """Populate tables."""
    ClinicalTrialPopulator().populate()
    DrugPopulator().populate()


class DrugPopulator:
    """Populate the associated drug tables."""

    def __init__(self):
        """Init method for DrugPopulator."""
        self.__num_drugs_in_graphstore = rest_query.sql(DRUG_COUNT).data[0]['num_drugs']

    def __collect_drugs_from_graphstore(self, chunk_size: int = 10000) -> list:
        """Query clinical trial data in chunks."""
        logger.warning("Collecting drug metadata from graphstore")
        total_num_chunks = (self.__num_drugs_in_graphstore // chunk_size) + 1

        chunk_index = 0
        while chunk_index < total_num_chunks:
            table_chunk = rest_query.sql(DRUG_METADATA.format(chunk_size * chunk_index, chunk_size)).data
            yield table_chunk
            chunk_index += 1

    @staticmethod
    def __extract_values(value_dict: dict, key: str):
        """Extract non-None values from key if present and pops key."""
        values = []
        if key in value_dict:
            vals = value_dict.pop(key)
            if vals is not None:
                for val in vals:
                    if val is not None:
                        values.append(val)

        return value_dict, values

    def populate(self):
        """Populate the SQLite DB with Drug metadata."""
        update = self.__update_needed()
        if update:
            logger.info("Populating drug information")
            self.__parse__and_import_drug_metadata()

    def __update_needed(self) -> bool:
        """Check if drug data is missing."""
        db_entries = sess.query(func.count(Drugs.id)).first()[0]
        return False if db_entries == self.__num_drugs_in_graphstore else True

    def __parse__and_import_drug_metadata(self):
        drug_generator = self.__collect_drugs_from_graphstore()
        logger.info("Parsing and importing drug information from graphstore")
        for drug_data_chunk in drug_generator:
            for drug_entry in tqdm(drug_data_chunk, desc="Parsing and importing drug data"):
                patents = drug_entry.pop("drug_patents")
                products = drug_entry.pop("drug_products")
                drug_entry, targets = self.__extract_values(drug_entry, "target_symbols")

                drug_entry, trial_table_values = self.__extract_values(drug_entry, "clinical_trials")

                patent_rows = self.__parse_patents(patents) if patents else []
                product_rows = self.__parse_products(products) if products else []

                drug_entry['num_targets'] = len(targets)
                drug_entry['targets'] = "|".join(targets)
                drug_entry['clinical_trials'] = "|".join(trial_table_values)

                new_drug = Drugs(**drug_entry)
                new_drug.patents = patent_rows
                new_drug.products = product_rows
                sess.add(new_drug)

        sess.commit()

    @staticmethod
    def __get_clinical_trial_ids() -> dict:
        """Get row entries and Clinical Trial IDs for each CT row in DB."""
        trial_rows = sess.query(Trials, Trials.trial_id).all()
        return {r[1]: r[0] for r in trial_rows}

    @staticmethod
    def __parse_patents(patents: Dict[str, dict]) -> list:
        """Parse patent information from graphstore and imports it into SQLite DB at the end."""
        patent_rows = []
        patent_info = patents['patent']

        if not isinstance(patent_info, list):
            patent_info = [patent_info]

        for indiv_patent in patent_info:

            expired = datetime.today() > datetime.strptime(indiv_patent['expires'], "%Y-%m-%d")  # Boolean
            pat_number = indiv_patent['number']

            import_data = {'expired': expired, 'patent_number': pat_number}
            new_patent = Patents(**import_data)
            patent_rows.append(new_patent)
            sess.add(new_patent)

        return patent_rows

    @staticmethod
    def __parse_products(products_raw: Union[Dict[str, dict], List[Dict[str, dict]]]) -> list:
        """Parse product information from graphstore and imports it into SQLite DB at the end."""
        product_rows = []

        if not isinstance(products_raw, list):
            products_raw = [products_raw]

        labelled_products = {product['name']: {'has_generic': False,
                                               'is_approved': False,
                                               'has_approved_generic': False}
                             for product in products_raw}

        for product in products_raw:  # Usually several disrtibutors for the same drug
            product_name = product['name']

            if product['approved'] == 'true':
                labelled_products[product_name]['is_approved'] = True

            if product['generic'] == 'true':
                labelled_products[product_name]['has_generic'] = True

            if labelled_products[product_name]['is_approved'] and labelled_products[product_name]['has_generic']:
                labelled_products[product_name]['has_approved_generic'] = True

        for product_name, metadata in labelled_products.items():
            import_metadata = {'product_name': product_name, **metadata}

            new_product = Products(**import_metadata)
            product_rows.append(new_product)
            sess.add(new_product)

        return product_rows


class ClinicalTrialPopulator:
    """Clinical Trials class for importing into DB."""

    def __init__(self):
        """Init method to CTPopulator."""
        self.__num_trials_in_graphstore = rest_query.sql(CLINICAL_TRIALS_COUNT).data[0]['trial_count']

    def populate(self):
        """Populate the SQLite DB with ClinicalTrial data."""
        update = self.__update_needed()
        if update:
            logger.info("Populating Clinical Trial information")
            ct_data = self.__collect_ct_info()
            self.__populate_table(ct_data)

    def __update_needed(self) -> bool:
        """Check if Clinical Trial data is missing."""
        db_entries = sess.query(func.count(Trials.id)).first()[0]
        return False if db_entries == self.__num_trials_in_graphstore else True

    def __collect_ct_info_from_graphstore(self, chunk_size: int = 10000) -> list:
        """Query clinical trial data in chunks."""
        total_num_chunks = (self.__num_trials_in_graphstore // chunk_size) + 1

        chunk_index = 0
        while chunk_index < total_num_chunks:
            table_chunk = rest_query.sql(CLINICAL_TRIALS_DATA.format(chunk_size * chunk_index, chunk_size)).data
            yield table_chunk
            chunk_index += 1

    def __collect_ct_info(self) -> list:
        """Collect clinical trial information for every identified drug."""
        num_chunks = (self.__num_trials_in_graphstore // 10000) + 1
        data_generator = self.__collect_ct_info_from_graphstore()

        to_import = []
        for data_chunk in tqdm(data_generator, total=num_chunks, desc="Collecting all CT data"):
            if data_chunk:
                for r in data_chunk:
                    # Parse graphstore data
                    status = r['overall_status']
                    trial_id = r['trial_id']
                    primary_condition = r['condition'] if r['condition'] is not None else []
                    mesh = r['mesh_conditions'] if r['mesh_conditions'] is not None else []
                    conditions = ";".join(set(primary_condition + mesh))
                    trial_drugs = r['drugs_in_trial'] if 'drugs_in_trial' in r else None

                    # Structure data
                    metadata = {'trial_id': trial_id, 'trial_status': status,
                                'conditions': conditions, 'drugs_in_trial': trial_drugs}

                    to_import.append(metadata)

        return to_import

    @staticmethod
    def __populate_table(data: list):
        """Populate the SQLite DB with ClinicalTrial data."""
        logger.info("Importing Clinical Trial data")
        for entry in tqdm(data, desc="Parsing and importing clinical trials data"):
            for key, vals in entry.items():
                if isinstance(vals, list):
                    entry[key] = "|".join(vals)

            row = Trials(**entry)
            sess.add(row)

        sess.commit()
